@@ -115,6 +115,19 @@ impl FEM {
             .filter_map(|x| x.as_ref())
             .fold(0usize, |a, x| a + x.len())
     }
+
+    /// Loads FEM static solution gain matrix from a pickle file "static_reduction_model.73.pkl" located in a directory given by the `FEM_REPO` environment variable
+    pub fn static_from_env(self) -> Self {
+        let fem_repo = env::var("FEM_REPO").unwrap();
+        println!("Loading static gain matrix from static_reduction_model.73.pkl...");
+        let fem_static =
+            Self::from_pickle(Path::new(&fem_repo).join("static_reduction_model.73.pkl")).unwrap();
+        Self {
+            static_gain: fem_static.static_gain,
+            ..self
+        }
+    }
+
     /// Selects the inputs according to their natural ordering
     pub fn keep_inputs(&mut self, id: &[usize]) -> &mut Self {
         self.inputs.iter_mut().enumerate().for_each(|(k, i)| {
@@ -230,6 +243,67 @@ impl FEM {
             })
             .collect()
     }
+    /// Returns the inputs 2 modes transformation matrix for a given input
+    pub fn input2modes(&self, id: usize) -> Option<Vec<f64>> {
+        self.inputs[id].as_ref().map(|input| {
+            let indices: Vec<u32> = input
+                .iter()
+                .filter_map(|x| match x {
+                    IO::On(io) => Some(io.indices.clone()),
+                    IO::Off(_) => None,
+                })
+                .flatten()
+                .collect();
+            let n = self.inputs_to_modal_forces.len() / self.n_modes();
+            self.inputs_to_modal_forces
+                .chunks(n)
+                .flat_map(|x| {
+                    indices
+                        .iter()
+                        .map(|i| x[*i as usize - 1])
+                        .collect::<Vec<f64>>()
+                })
+                .collect()
+        })
+    }
+    pub fn trim2input(&self, id: usize, matrix: &na::DMatrix<f64>) -> Option<na::DMatrix<f64>> {
+        assert_eq!(
+            matrix.ncols(),
+            self.n_inputs(),
+            "Matrix columns # do not match inputs #"
+        );
+        self.inputs[id].as_ref().map(|input| {
+            let indices: Vec<u32> = input
+                .iter()
+                .filter_map(|x| match x {
+                    IO::On(io) => Some(io.indices.clone()),
+                    IO::Off(_) => None,
+                })
+                .flatten()
+                .collect();
+            na::DMatrix::from_columns(
+                &indices
+                    .iter()
+                    .map(|&i| matrix.column(i as usize - 1))
+                    .collect::<Vec<_>>(),
+            )
+        })
+    }
+    /// Returns the inputs 2 modes transformation matrix for an input type
+    pub fn in2modes<U>(&self) -> Option<Vec<f64>>
+    where
+        Vec<Option<fem_io::Inputs>>: fem_io::FemIo<U>,
+    {
+        <Vec<Option<fem_io::Inputs>> as fem_io::FemIo<U>>::position(&self.inputs)
+            .and_then(|id| self.input2modes(id))
+    }
+    pub fn trim2in<U>(&self, matrix: &na::DMatrix<f64>) -> Option<na::DMatrix<f64>>
+    where
+        Vec<Option<fem_io::Inputs>>: fem_io::FemIo<U>,
+    {
+        <Vec<Option<fem_io::Inputs>> as fem_io::FemIo<U>>::position(&self.inputs)
+            .and_then(|id| self.trim2input(id, matrix))
+    }
     /// Returns the modes 2 outputs transformation matrix for the turned-on outputs
     pub fn modes2outputs(&mut self) -> Vec<f64> {
         let n = self.n_modes();
@@ -264,6 +338,42 @@ impl FEM {
                 .collect()
         })
     }
+    pub fn trim2output(&self, id: usize, matrix: &na::DMatrix<f64>) -> Option<na::DMatrix<f64>> {
+        assert_eq!(
+            matrix.nrows(),
+            self.n_outputs(),
+            "Matrix rows # do not match outputs #"
+        );
+        //let q: Vec<_> = matrix.chunks(self.n_modes()).collect();
+        self.outputs[id].as_ref().map(|output| {
+            na::DMatrix::from_rows(
+                &output
+                    .iter()
+                    .filter_map(|x| match x {
+                        IO::On(io) => Some(io.indices.clone()),
+                        IO::Off(_) => None,
+                    })
+                    .flatten()
+                    .map(|i| matrix.row(i as usize - 1))
+                    .collect::<Vec<_>>(),
+            )
+        })
+    }
+    /// Returns the modes 2 outputs transformation matrix for an output type
+    pub fn modes2out<U>(&self) -> Option<Vec<f64>>
+    where
+        Vec<Option<fem_io::Outputs>>: fem_io::FemIo<U>,
+    {
+        <Vec<Option<fem_io::Outputs>> as fem_io::FemIo<U>>::position(&self.outputs)
+            .and_then(|id| self.modes2output(id))
+    }
+    pub fn trim2out<U>(&self, matrix: &na::DMatrix<f64>) -> Option<na::DMatrix<f64>>
+    where
+        Vec<Option<fem_io::Outputs>>: fem_io::FemIo<U>,
+    {
+        <Vec<Option<fem_io::Outputs>> as fem_io::FemIo<U>>::position(&self.outputs)
+            .and_then(|id| self.trim2output(id, matrix))
+    }
     /// Return the static gain reduced to the turned-on inputs and outputs
     pub fn reduced_static_gain(&mut self, n_io: (usize, usize)) -> Option<na::DMatrix<f64>> {
         let n_reduced_io = (self.n_inputs(), self.n_outputs());
@@ -318,9 +428,12 @@ impl FEM {
             na::DMatrix::from_row_slice(self.n_outputs(), self.n_modes(), &self.modes2outputs());
         let d = na::DMatrix::from_diagonal(
             &na::DVector::from_row_slice(&self.eigen_frequencies_to_radians())
-                .map(|x| 1f64 / (x * x)),
+                .map(|x| 1f64 / (x * x))
+                .remove_rows(0, 3),
         );
-        modes_2_nodes * d * forces_2_modes
+
+        // println!("{ }",d.fixed_slice::<3,3>(0,0)); <- Just checking if unstable modes were removed
+        modes_2_nodes.remove_columns(0, 3) * d * forces_2_modes.remove_rows(0, 3)
     }
 }
 impl fmt::Display for FEM {
